/*
 * Copyright (C) 2022 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */

package inspect

import (
	"encoding/json"
	"errors"
	"fmt"
	"io/ioutil"
	"log"
	"os"
	wf "ssa/seci_ingestion_status/pkg/wf_info"
	"strconv"
	"strings"
)

type IngestionStatus struct {
	Type       string
	Subject    `json:"subject"`
	IsIngested bool
	Status     string
}

type Subject struct {
	WfId       int    `json:"workflow_request_id"`
	WfName     string `json:"workflow_name"`
	ResultsDir string `json:"results_dir"`
	Argument   `json:"argument"`
}

type Argument struct {
	CachePath string `json:"cache_path"`
}

/**
 * Check if an error happened and panic on it if so
 *
 * @param err An error object to report
 **/
func checkError(err error) {
	if err != nil {
		panic(err)
	}
}

/**
 * Inspect a SECI Ingestion Workflow Request to determine status
 *
 * @param WfInfo A struct with needed workflow information
 * @return a formatted string containing the status of a given SECI Ingestion Workflow
 **/
func InspectSECIIngestionStatus(wf_info wf.WfInfo) string {
	var statusFile string = "ingest-result.json"
	var ingest_status IngestionStatus
	ingest_status.IsIngested = false
	ingest_status.Status = "Not Ingested"

	file_content, err := ioutil.ReadFile(fmt.Sprintf("%s/%s", wf_info.ResultsDir, statusFile))
	if errors.Is(err, os.ErrNotExist) {
		ingest_status.Status = "Unknown"
		return generateStatusMsg(ingest_status)
	} else {
		checkError(err)
	}

	err = json.Unmarshal(file_content, &ingest_status)
	if err != nil {
		log.Fatal("Error during Unmarshal(): ", err)
	}

	if ingest_status.Type == "workflow-complete" {
		ingest_status.IsIngested = true
		ingest_status.Status = "Ingested"
	}

	return generateStatusMsg(ingest_status)
}

func generateStatusMsg(s IngestionStatus) string {
	var additionalInfo string = ""
	if s.Status == "Unknown" {
		additionalInfo = fmt.Sprintf("\n\nUnable to determine ingestion status. Ingestion may still be running or the ingestion workflow request may not have been created")
	} else if !s.IsIngested {
		additionalInfo = fmt.Sprintf("\n\nLook in %s for more information.", s.ResultsDir)
	} else {
		additionalInfo = fmt.Sprintf("\n\nWorkflow: %s has been ingested", strconv.Itoa(s.WfId))
	}
	return fmt.Sprintf(
		"Workflow Request Id: %s\nWorkflow Name: %s\nResults Directory: %s\nCache Path: %s\n\nStatus: %s",
		strconv.Itoa(s.WfId),
		s.WfName,
		s.ResultsDir,
		s.CachePath,
		strings.ToUpper(s.Status),
	) + additionalInfo
}
