/*
 * Copyright (C) 2022 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */

package wf_info

import (
	"encoding/json"
	"fmt"
	"net/http"

	"github.com/magiconair/properties"
)

type WfInfo struct {
	WfId       int64  `json:"workflow_request_id"`
	WfName     string `json:"workflow_name"`
	ResultsDir string `json:"results_dir"`
}

/**
 * Check if an error happened and panic on it if so
 *
 * @param err An error object to report
 **/
func checkError(err error) {
	if err != nil {
		panic(err)
	}
}

/**
 * Pull a property from a CAPO property file
 *
 * @param capo_profile A string with a path to the CAPO property file
 * @param property_name A string with the name of the property to be returned
 * @return a string with the property value
 **/
func getCapoProperty(capo_profile string, property_name string) string {
	prop_file := properties.MustLoadFile(capo_profile, properties.UTF8)

	property := prop_file.GetString(property_name, "")
	if len(property) == 0 {
		panic(fmt.Sprintf("Unable to retrieve property %s from %s", property_name, capo_profile))
	}

	return property
}

/**
 * Get Workflow info for SECI Ingestions
 *
 * @param wf_req_id A string of a workflow request id
 * @param capo_profile A string with the path to the CAPO property file used to
 * get the workflow service url
 * @return a WfInfo struct
 **/
func GetSECIWorkflowInfo(wf_req_id string, capo_profile string) WfInfo {
	workflow_url_prop := "edu.nrao.workspaces.WorkflowSettings.serviceUrl"
	workflow_url := getCapoProperty(capo_profile, workflow_url_prop)

	workflow_endpoint := fmt.Sprintf("%s/workflows/ingest_seci/requests/%s", workflow_url, wf_req_id)

	resp, err := http.Get(workflow_endpoint)
	checkError(err)

	if resp.StatusCode != http.StatusOK {
		panic(fmt.Sprintf("Error making GET request to URL %s, got status code %d", workflow_endpoint, resp.StatusCode))
	}

	defer resp.Body.Close()

	wf_info := new(WfInfo)
	err = json.NewDecoder(resp.Body).Decode(wf_info)
	checkError(err)

	if wf_info.WfName != "ingest_seci" {
		panic(fmt.Sprintf("The provided workflow request id %s is not an ingest_seci workflow. Cannot provide status.", wf_req_id))
	}

	return *wf_info
}
